@@ -433,6 +433,11 @@ type ModelValidationSet struct {
 	Mode ModelValidationSetMode
 }
 
+// SequenceKey returns the sequence key for this validation set.
+func (mvs *ModelValidationSet) SequenceKey() string {
+	return vsSequenceKey(release.Series, mvs.AccountID, mvs.Name)
+}
+
 func (mvs *ModelValidationSet) AtSequence() *AtSequence {
 	return &AtSequence{
 		Type:        ValidationSetType,
@@ -608,6 +613,15 @@ func (mod *Model) SnapsWithoutEssential() []*ModelSnap {
 	return mod.allSnaps[mod.numEssentialSnaps:]
 }
 
+// AllSnaps returns all the snaps listed by the model, across all modes.
+// Essential snaps are at the front of the slice, followed by the non-essential
+// snaps. The essential snaps follow the same order as returned by
+// EssentialSnaps. The non-essential snaps are returned in the order they are
+// mentioned in the model.
+func (mod *Model) AllSnaps() []*ModelSnap {
+	return mod.allSnaps
+}
+
 // ValidationSets returns all the validation-sets listed by the model.
 func (mod *Model) ValidationSets() []*ModelValidationSet {
 	return mod.validationSets
