@@ -78,6 +78,8 @@ const (
 func setupConfCacheDirs(newrootdir string) {
 	ConfDir = filepath.Join(newrootdir, "/etc/apparmor.d")
 	CacheDir = filepath.Join(newrootdir, "/var/cache/apparmor")
+	hostAbi30File = filepath.Join(newrootdir, "/etc/apparmor.d/abi/3.0")
+	hostAbi40File = filepath.Join(newrootdir, "/etc/apparmor.d/abi/4.0")
 
 	SystemCacheDir = filepath.Join(ConfDir, "cache")
 	exists, isDir, _ := osutil.DirExists(SystemCacheDir)
@@ -209,6 +211,14 @@ var (
 	// Filesystem root defined locally to avoid dependency on the
 	// 'dirs' package
 	rootPath = "/"
+
+	// hostAbi30File is the path to the apparmor "3.0" ABI file and is typically
+	// /etc/apparmor.d/abi/3.0. It is not present on all systems. It is notably
+	// absent when using apparmor 2.x. The variable reacts to changes to global
+	// root directory.
+	hostAbi30File = ""
+	// hostAbi40File is like hostAbi30File but for ABI 4.0
+	hostAbi40File = ""
 )
 
 // Each apparmor feature is manifested as a directory entry.
@@ -345,6 +355,16 @@ func probeKernelFeatures() ([]string, error) {
 	for _, fi := range dentries {
 		if fi.IsDir() {
 			features = append(features, fi.Name())
+			// also read any sub-features
+			subdenties, err := ioutil.ReadDir(filepath.Join(rootPath, featuresSysPath, fi.Name()))
+			if err != nil {
+				return []string{}, err
+			}
+			for _, subfi := range subdenties {
+				if subfi.IsDir() {
+					features = append(features, fi.Name()+":"+subfi.Name())
+				}
+			}
 		}
 	}
 	return features, nil
@@ -353,6 +373,7 @@ func probeKernelFeatures() ([]string, error) {
 func probeParserFeatures() ([]string, error) {
 	var featureProbes = []struct {
 		feature string
+		flags   []string
 		probe   string
 	}{
 		{
@@ -387,6 +408,11 @@ func probeParserFeatures() ([]string, error) {
 			feature: "userns",
 			probe:   "userns,",
 		},
+		{
+			feature: "unconfined",
+			flags:   []string{"unconfined"},
+			probe:   "# test unconfined",
+		},
 	}
 	_, internal, err := AppArmorParser()
 	if err != nil {
@@ -396,7 +422,7 @@ func probeParserFeatures() ([]string, error) {
 	for _, fp := range featureProbes {
 		// recreate the Cmd each time so we can exec it each time
 		cmd, _, _ := AppArmorParser()
-		if tryAppArmorParserFeature(cmd, fp.probe) {
+		if tryAppArmorParserFeature(cmd, fp.flags, fp.probe) {
 			features = append(features, fp.feature)
 		}
 	}
@@ -470,6 +496,23 @@ func AppArmorParser() (cmd *exec.Cmd, internal bool, err error) {
 	for _, dir := range filepath.SplitList(parserSearchPath) {
 		path := filepath.Join(dir, "apparmor_parser")
 		if _, err := os.Stat(path); err == nil {
+			// Detect but ignore apparmor 4.0 ABI support.
+			//
+			// At present this causes some bugs with mqueue mediation that can
+			// be avoided by pinning to 3.0 (which is also supported on
+			// apparmor 4). Once the mqueue issue is analyzed and fixed, this
+			// can be replaced with a --policy-features=hostAbi40File pin like
+			// we do below.
+			if fi, err := os.Lstat(hostAbi40File); err == nil && !fi.IsDir() {
+				logger.Debugf("apparmor 4.0 ABI detected but ignored")
+			}
+
+			// Perhaps 3.0?
+			if fi, err := os.Lstat(hostAbi30File); err == nil && !fi.IsDir() {
+				return exec.Command(path, "--policy-features", hostAbi30File), false, nil
+			}
+
+			// Most likely 2.0
 			return exec.Command(path), false, nil
 		}
 	}
@@ -478,9 +521,14 @@ func AppArmorParser() (cmd *exec.Cmd, internal bool, err error) {
 }
 
 // tryAppArmorParserFeature attempts to pre-process a bit of apparmor syntax with a given parser.
-func tryAppArmorParserFeature(cmd *exec.Cmd, rule string) bool {
+func tryAppArmorParserFeature(cmd *exec.Cmd, flags []string, rule string) bool {
 	cmd.Args = append(cmd.Args, "--preprocess")
-	cmd.Stdin = bytes.NewBufferString(fmt.Sprintf("profile snap-test {\n %s\n}", rule))
+	flagSnippet := ""
+	if len(flags) > 0 {
+		flagSnippet = fmt.Sprintf("flags=(%s) ", strings.Join(flags, ","))
+	}
+	cmd.Stdin = bytes.NewBufferString(fmt.Sprintf("profile snap-test %s{\n %s\n}",
+		flagSnippet, rule))
 	output, err := cmd.CombinedOutput()
 	// older versions of apparmor_parser can exit with success even
 	// though they fail to parse
